using System.Text;
using Luski.Shared.PublicServers.V1.Enums;
using Luski.Shared.PublicServers.V1.ServerToClient.HTTP;
using Luski.Shared.PublicServers.V1.Shared;
using LuskiServer.Classes.TableDef;
using LuskiServer.Enums;
using LuskiServer.Interfaces;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Classes;

public static class EXT
{
    public static string GetNumberString(this ServerPermission enu)
    {
       return ((long)(enu)).ToString();
    }
    
    public static ServerPermission GetPermission(this string enu)
    {
        return (ServerPermission)long.Parse(enu);
    }
    public static bool GetBool(this string enu)
    {
        if (enu == "0") return false;
        return true;
    }
    
    public static bool CanTokenRequest(this ControllerBase Base, out long id, out long sid, out IActionResult? result)
    {
        if (Base.Request.Headers.ContainsKey("token"))
        {
            return ValidateToke(Base, out id, out sid, out result);
        }
        result = Base.ShowError(ErrorCode.MissingToken, "You did not provide a token");
        id = 0;
        sid = 0;
        return false;
    }

    public static IActionResult ShowError(this ControllerBase Base, Exception Error, bool LogInDB = true)
    {
        if (LogInDB)
        {
            Tables.Logs.Insert(
                Logs.ID.CreateParameter(LuskiFunctions.Snowflake.GenerateSnowflake(LuskiFunctions.Config.ServerEpoch).ID), 
                Logs.Type.CreateParameter(LogType.Error),
                Logs.Message.CreateParameter(Error.ToString() + " in " + Base.Url.ToString()));
        }

        return Base.ShowError(ErrorCode.ServerError, Error.Message);
    }
    
    public static IActionResult ResponseCodeToResult(this ControllerBase Base, ErrorCode Result, string msg = "")
    {
        return Base.ResponseToResult(
            new STC()
            {
                Error = Result,
                ErrorMessage = msg
            });
    }

    public static IActionResult ResponseToResult<TResult>(this ControllerBase Base, TResult Result) where TResult : STC
    {
        return Base.StatusCode(StatusCodes.Status200OK, Result);
    }
    
    public static IActionResult ShowError(this ControllerBase Base, ErrorCode code, string Error)
    {
        int cod = 403;
        if (code == ErrorCode.ServerError) cod = StatusCodes.Status500InternalServerError;
        return Base.StatusCode(cod, new STC()
        {
            Error = code,
            ErrorMessage = Error
        });
    }

    private static bool CheckToken(string Token, ref long ID, ref long SID)
    {
        try
        {
            string id = Encoding.UTF8.GetString(Convert.FromBase64String(Token.Split('.')[1]));
            string? tok = Tables.Sessions.Read(Sessions.Token, Sessions.ID.CreateParameter(long.Parse(id)));
            if (!string.IsNullOrWhiteSpace(tok) && tok == Token)
            {
                SID = long.Parse(id);
                ID = Tables.Sessions.Read(Sessions.User, Sessions.ID.CreateParameter(SID));
                return true;
            }
            return false;
        }
        catch
        {
            return false;
        }
    }
    
    private static bool ValidateToke(ControllerBase Base, out long Id, out long Sid, out IActionResult? result)
    {
        long id = 0, sid = 0;
        if (CheckToken(Base.Request.Headers["token"].First()!, ref id, ref sid))
        {
            result = null;
            Id = id;
            Sid = sid;
            return true;
        }
        Id = id;
        Sid = sid;
        result = Base.ShowError(ErrorCode.InvalidToken, (string)null!);
        return false;
    }
}