using System.Net;
using System.Runtime.InteropServices.JavaScript;
using System.Text;
using System.Text.Json;
using System.Text.Json.Serialization;
using LuskiServer.Classes.TableDef;
using LuskiServer.Enums.ServerComs;
using LuskiServer.Interfaces.ServerComs;
using WebSocketSharp;
using WebSocketSharp.Server;

namespace LuskiServer.Classes.ServerComs;

public static class WSS
{
    public static WebSocketServer ServerIPv4 = null!, ServerIPv6 = null!;
    public class CoreV1 : WebSocketBehavior
    {
        protected override void OnClose(CloseEventArgs e)
        {
            try
            {
                SessionsRow id = Tables.Sessions.ReadRow(TableDef.Sessions.WSSTCP.CreateParameter(ID));
                Tables.Sessions.DeleteRow(TableDef.Sessions.ID.CreateParameter(id.ID));
                if (id.SessionKey != 0)
                {
                    Tables.Keys.DeleteRow(Keys.ID.CreateParameter(id.SessionKey));
                }
            }
            catch (Exception exception)
            {
                Console.WriteLine(exception);
            }
            base.OnClose(e);
        }

        protected override void OnMessage(MessageEventArgs e)
        {
            ServerSendSendEvent? BaseMessage = JsonSerializer.Deserialize(e.Data, ServerSendSendEventContext.Default.ServerSendSendEvent);
            if (BaseMessage is not null)
            {
                switch (BaseMessage.SendType)
                {
                    case SendType.All:
                        switch (BaseMessage.Data.Type)
                        {
                            case DataType.MessageCreate:
                                MessageEvent msg = (BaseMessage.Data.Data as MessageEvent)!;
                                break;
                        }
                        break;
                    case SendType.ID_Group:
                        switch (BaseMessage.Data.Type)
                        {
                            case DataType.MessageCreate:
                                MessageEvent msg = (BaseMessage.Data.Data as MessageEvent)!;
                                string raw = JsonSerializer.Serialize(msg, MessageEventContext.Default.MessageEvent);
                                foreach (long user in BaseMessage.IDS!)
                                {
                                    if (Tables.Sessions.TryReadRows(out SessionsRow[] rows, TableDef.Sessions.User.CreateParameter(user)))
                                    {
                                        foreach (SessionsRow row in rows)
                                        {
                                            Sessions.SendTo(raw, row.WSSTCP);
                                        }
                                    }
                                }
                                break;
                        }
                        break;
                    case SendType.Client:
                        switch (BaseMessage.Data.Type)
                        {
                            case DataType.Token:
                                TokenEvent te = JsonSerializer.Deserialize(BaseMessage.Data.Data.ToString()!, TokenEventContext.Default.TokenEvent)!;
                                if (Tables.Sessions.TryReadRow(out SessionsRow row,
                                        TableDef.Sessions.LoginToken.CreateParameter(te.Token)))
                                {
                                    string Token = $"{Convert.ToBase64String(Encoding.UTF8.GetBytes(row.User.ToString()))}.{Convert.ToBase64String(Encoding.UTF8.GetBytes(row.ID.ToString()))}.{Convert.ToBase64String(Encoding.UTF8.GetBytes(DateTime.UtcNow.ToString()))}.{Convert.ToBase64String(Encoding.UTF8.GetBytes(new Random().Next(0, 1000000).ToString()))}.{Convert.ToBase64String(Encoding.UTF8.GetBytes(new Random().Next(0, 1000000).ToString()))}";
                                    te.Token = Token;
                                    te.Type = DataType.Token;
                                    Tables.Sessions.Update(TableDef.Sessions.ID, row.ID, TableDef.Sessions.LoginToken.CreateParameter(string.Empty), TableDef.Sessions.Token.CreateParameter(Token), TableDef.Sessions.WSSTCP.CreateParameter(ID));
                                    Token = $"{Convert.ToBase64String(Encoding.UTF8.GetBytes(row.User.ToString()))}.{Convert.ToBase64String(Encoding.UTF8.GetBytes(DateTime.UtcNow.ToString()))}.{Convert.ToBase64String(Encoding.UTF8.GetBytes(new Random().Next(0, 1000000).ToString()))}.{Convert.ToBase64String(Encoding.UTF8.GetBytes(new Random().Next(0, 1000000).ToString()))}";
                                    te.SessionToken = Token;
                                    Tables.SessionTokens.Insert(
                                        SessionTokens.Token.CreateParameter(Token),
                                        SessionTokens.AccountID.CreateParameter(row.User),
                                        SessionTokens.TimeFilter.CreateParameter(DateTime.UtcNow.AddDays(5)),
                                        SessionTokens.AddressFilter.CreateParameter(Array.Empty<byte>()),
                                        SessionTokens.ID.CreateParameter(LuskiFunctions.Snowflake.GenerateSnowflake(LuskiFunctions.Config.ServerEpoch).ID));
                                    string raw = JsonSerializer.Serialize(te, TokenEventContext.Default.TokenEvent);
                                    Send(raw);
                                }
                                break;
                        }
                        break;
                    default:
                        break;
                }
            }
            base.OnMessage(e);
        }
    }

    private static string path = "";

    public static void Init(string IPv4, int IPv4_Port, string IPv4_URL, bool IPv4_Secure = true, string? IPv6 = null, string? IPv6_URL = null, int IPv6_Port = 0, bool IPv6_Secure = true)
    {
        ServerIPv4 = new WebSocketServer(IPAddress.Parse(IPv4), IPv4_Port, IPv4_Secure);
        ServerIPv4.AddWebSocketService<CoreV1>($"{IPv4_URL}v1");
        path = $"{IPv4_URL}v1";
        ServerIPv4.Start();
        if (IPv6 is not null)
        {
            ServerIPv6 = new WebSocketServer(IPAddress.Parse(IPv6), IPv6_Port, IPv6_Secure);
            ServerIPv6.AddWebSocketService<CoreV1>($"{IPv6_URL}v1");
            ServerIPv6.Start();
        }
    }
    
    public static void SendData(SendType SendType, ServerEvent Event, params long[] IDS)
    {
        ServerSendSendEvent SSSE = new()
        {
            SendType = SendType,
            Data = Event
        };
        if (SendType == SendType.ID_Group) SSSE.IDS = IDS;
        else if (SendType == SendType.ID) SSSE.ID = IDS[0];

        if (SendType == SendType.ID_Group)
        {
            foreach (long id in IDS)
            {
                Console.WriteLine("Trying To Find: " + id);
                bool sec = Tables.Sessions.TryReadRows(out SessionsRow[] srs, Sessions.User.CreateParameter(id));
                if (sec)
                {
                    foreach (SessionsRow sr in srs)
                    {
                        Console.WriteLine("Trying To Send Data To: " + id + " as " + sr.WSSTCP);
                        string d = JsonSerializer.Serialize(Event);
                        Console.WriteLine("Sending Data: \n" + d);
                        ServerIPv4.WebSocketServices[path].Sessions.SendTo(d, sr.WSSTCP);
                    }
                }
            }
        }
        
        //TODO Start work on server send code
    }
}