using Asp.Versioning;
using Luski.Shared.PublicServers.V1.ClientToServer.HTTP;
using Luski.Shared.PublicServers.V1.Enums;
using Luski.Shared.PublicServers.V1.ServerToClient.HTTP;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class OfflineDataController : ControllerBase
{
    [HttpGet]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Get([FromHeader(Name = "token")]string? token, [FromHeader(Name = "storage_id")]long SID)
    {
        try
        {
            Console.WriteLine("Storage GET 1: " +SID.ToString());
            if (!this.CanTokenRequest(out long UID, out long SSID, out IActionResult? toc) && toc != null) return toc;
            Console.WriteLine("Storage GET 1: " +SID.ToString());
            if (!Tables.Storage.TryReadRow(out StorageRow row, Storage.ID.CreateParameter(SID)))
                return this.ResponseCodeToResult(ErrorCode.InvalidHeader);
            if (row.AccountID != UID) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            List<string> bbb = new();
            foreach (byte[]? blob in row.Data)
            {
                bbb.Add(Convert.ToBase64String(blob));
            }
            Tables.Storage.Update(Storage.ID, SID, Storage.Data.CreateParameter(Array.Empty<byte[]>()));
            return this.ResponseToResult(new OfflineDataBlobSTC()
            {
                Data = bbb.ToArray()
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpGet]
    [Route(LuskiFunctions.Info.Routes.Default.Base + "/Info")]
    public IActionResult GetInfo([FromHeader(Name = "token")]string? token, [FromHeader(Name = "storage_id")]long SID)
    {
        try
        {
            foreach (var h in HttpContext.Request.Headers)
            {
                foreach (var v in h.Value)
                {
                    Console.WriteLine(h.Key + ": " + v);
                }
            }
            Console.WriteLine("Storage GET: " +SID.ToString());
            if (!this.CanTokenRequest(out long UID, out long SSID, out IActionResult? toc) && toc != null) return toc;
            Console.WriteLine("Storage GET: " +SID.ToString());
            if (!Tables.Storage.TryReadRow(out StorageRow row, Storage.ID.CreateParameter(SID)))
                return this.ResponseCodeToResult(ErrorCode.InvalidHeader);
            if (row.AccountID != UID) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            bool update = false;
            if (Tables.Sessions.Read(Sessions.StorageID, Sessions.ID.CreateParameter(SSID)) == 0)
            {
                SessionsRow[] rows = Tables.Sessions.ReadRows(uint.MaxValue, Sessions.ID.CreateParameter(SSID), Sessions.StorageID.CreateParameter(SID));
                if (rows.Length == 0)
                {
                    update = true;
                }
                Tables.Sessions.Update(Sessions.ID, SSID, Sessions.StorageID.CreateParameter(SID));
            }
            return this.ResponseToResult(new StorageInfoSTC()
            {
                Password = Convert.ToBase64String(row.Password),
                ID = row.ID,
                Update = update
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpPatch]
    [Route(LuskiFunctions.Info.Routes.Default.Base + "/Info")]
    public IActionResult SetInfo([FromBody]StorageInfoCTS si, [FromHeader(Name = "token")]string? token, [FromHeader(Name = "storage_id")]long SID)
    {
        try
        {
            foreach (var h in HttpContext.Request.Headers)
            {
                foreach (var v in h.Value)
                {
                    Console.WriteLine(h.Key + ": " + v);
                }
            }
            Console.WriteLine("Storage Patch: " +SID.ToString());
            if (!this.CanTokenRequest(out long UID, out long SSID, out IActionResult? toc) && toc != null) return toc;
            Console.WriteLine("Storage Patch: " +SID.ToString());
            if (!Tables.Storage.TryReadRow(out StorageRow row, Storage.ID.CreateParameter(SID)))
                return this.ResponseCodeToResult(ErrorCode.InvalidHeader);
            if (row.AccountID != UID) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            Tables.Storage.Update(Storage.ID, SID, Storage.Password.CreateParameter(Convert.FromBase64String(si.Password)));
            
            return this.ResponseToResult(new StorageInfoSTC()
            {
                Password = si.Password,
                ID = row.ID,
                Update = false
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpPost]
    [Route(LuskiFunctions.Info.Routes.Default.Base + "/Info")]
    public IActionResult PostInfo([FromBody]StorageInfoCTS si, [FromHeader(Name = "token")]string? token)
    {
        try
        {
            Console.WriteLine("POST");
            if (!this.CanTokenRequest(out long UID, out long SSID, out IActionResult? toc) && toc != null) return toc;
            LuskiFunctions.Snowflake sf = LuskiFunctions.Snowflake.GenerateSnowflake(LuskiFunctions.Config.ServerEpoch);
            Tables.Storage.Insert(
                Storage.ID.CreateParameter(sf.ID),
                Storage.Password.CreateParameter(Convert.FromBase64String(si.Password)),
                Storage.AccountID.CreateParameter(UID),
                Storage.Data.CreateParameter(Array.Empty<byte[]>()),
                Storage.OffileKey.CreateParameter(0));
            Tables.Sessions.Update(Sessions.StorageID, sf.ID, Sessions.ID.CreateParameter(SSID));
            
            return this.ResponseToResult(new StorageInfoSTC()
            {
                Password = si.Password,
                ID = sf.ID,
                Update = false
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
}