using Asp.Versioning;
using Luski.Shared.PublicServers.V1.Enums;
using Luski.Shared.PublicServers.V1.ServerToClient.HTTP;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class SocketBulkMessageController : ControllerBase
{
    [HttpGet]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Get()
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!Request.Headers.ContainsKey("id")) return this.ShowError(ErrorCode.MissingHeader, "You did not give an channel id for the channel you want to get a message from");
            string channel_id = Request.Headers["id"].First();
            long MessagesToDownload = 50;
            long Max = 200;
            if (long.TryParse(channel_id, out long Channel_Id) && !Tables.Channels.TryRead(Channels.ID, out _, Channels.ID.CreateParameter(Channel_Id))) return this.ShowError(ErrorCode.InvalidHeader, "The the channel id you have given the server is not a valed id");
            if (!LuskiFunctions.HasAccessToChannel(ID, Channel_Id, ServerPermission.ReadMessageHistory))
                return this.ResponseCodeToResult(ErrorCode.Forbidden);
            if (Request.Headers.ContainsKey("messages")) _ = long.TryParse(Request.Headers["messages"].First(), out MessagesToDownload);
            if (MessagesToDownload > Max) MessagesToDownload = Max;
            string? MostRecentID = null;
            string sign = "<";
            if (!Request.Headers.ContainsKey("mostrecentid") || !long.TryParse(MostRecentID = Request.Headers["mostrecentid"].First(), out long mrid) || !Tables.Messages.TryRead(Messages.ID, out _, Messages.ID.CreateParameter(mrid), Messages.ChannelID.CreateParameter(Channel_Id)))
            {
                try
                {
                    bool HasMRID = Tables.Messages.TryRead(Messages.ID, Messages.ID.GetDecendingOrder(), out long MRID, Messages.ChannelID.CreateParameter(Channel_Id));
                    if (HasMRID)
                    {
                        MostRecentID = MRID.ToString();
                        sign += "=";
                    }
                }
                catch
                {
                    return this.ResponseToResult(
                        new SocketBulkMessageSTC()
                        {
                            Messages = Array.Empty<MessageSTC>()
                        });
                }
            }
            SocketBulkMessageSTC all = new();
            List<MessageSTC> mmmm = new List<MessageSTC>();
            if (MostRecentID != null)
            {
                MessageRow[] rawmsgs = Tables.Messages.ReadRows((uint)MessagesToDownload,
                    Messages.ID.GetDecendingOrder(),
                    Messages.ID.CreateParameter(long.Parse(MostRecentID), sign),
                    Messages.ChannelID.CreateParameter(Channel_Id));
                
                foreach (MessageRow Row in rawmsgs)
                {
                    MessageSTC temp = new()
                    {
                        Context = Convert.ToBase64String(Row.Context),
                        EncoderType = Row.EncoderType,
                        EncryptionKey = Row.EncryptionKey,
                        ChannelID = Row.ChannelID,
                        AuthorID = Row.AuthorID,
                        ID = Row.ID,
                        Files = Array.Empty<ServerFileInfoSTC>(),
                        IsProfile = Row.IsChannelProfile,
                    };
                    if (Row.Files.Any())
                    {
                        List<ServerFileInfoSTC> list = new();
                        foreach (long b in Row.Files)
                        {
                            FileRow fileraw = Tables.Files.ReadRow(Files.ID.CreateParameter(b));
                            ServerFileInfoSTC file = new()
                            {
                                ID = b,
                                Name = Convert.ToBase64String(fileraw.Name),
                                Size = fileraw.Size,
                                NameEncoder = fileraw.NameEncoderType,
                                NameKey = fileraw.NameEncryptionKey,
                                Encoder = fileraw.EncoderType,
                                Key = fileraw.EncryptionKey,
                            };
                            list.Add(file);
                        }

                        temp.Files = list.ToArray();
                    }
                    mmmm.Add(temp);
                }
            }

            if (mmmm is not null) all.Messages = mmmm.ToArray();
            else all.Messages = Array.Empty<MessageSTC>();
            return this.ResponseToResult(all);
        }
        catch (Exception ex)
        {
            return this.ShowError(ex);
        }
    }
}