using Asp.Versioning;
using Luski.Shared.PublicServers.V1.ClientToServer.HTTP;
using Luski.Shared.PublicServers.V1.Enums;
using Luski.Shared.PublicServers.V1.ServerToClient.HTTP;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class SocketChannelController : ControllerBase
{
    [HttpPost]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Post([FromBody]ChannelPostCTS ChanReq)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!LuskiFunctions.HasAccessToCategory(ID, ChanReq.Parent, ServerPermission.CreateChannels)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            DateTime epoch = LuskiFunctions.Config.ServerEpoch;
            List<long> RoleOverides = new(), UserOverides = new();
            if (ChanReq.RoleOverrides.Length > 0)
            {
                foreach (UserRoleOverrideCTS over in ChanReq.RoleOverrides)
                {
                    LuskiFunctions.Snowflake overSnowflake = LuskiFunctions.Snowflake.GenerateSnowflake(epoch);
                    Tables.ServerRoleOverrides.Insert(
                        ServerRoleOverrides.ID.CreateParameter(overSnowflake.ID),
                        ServerRoleOverrides.RoleID.CreateParameter(over.RoleID),
                        ServerRoleOverrides.GoodPermissions.CreateParameter(over.GoodPermissions),
                        ServerRoleOverrides.BadPermissions.CreateParameter(over.BadPermissions));
                    RoleOverides.Add(overSnowflake.ID);
                }
            }
            if (ChanReq.UserOverrides.Length > 0)
            {
                foreach (UserOverrideCTS over in ChanReq.UserOverrides)
                {
                    LuskiFunctions.Snowflake overSnowflake = LuskiFunctions.Snowflake.GenerateSnowflake(epoch);
                    Tables.UserRoleOverrides.Insert(
                        UserRoleOverrides.ID.CreateParameter(overSnowflake.ID),
                        UserRoleOverrides.UserID.CreateParameter(over.UserID),
                        UserRoleOverrides.GoodPermissions.CreateParameter(over.GoodPermissions),
                        UserRoleOverrides.BadPermissions.CreateParameter(over.BadPermissions));
                    UserOverides.Add(overSnowflake.ID);
                }
            }

            epoch = DateTime.UtcNow.Date;
            LuskiFunctions.Snowflake chanSnowflake = LuskiFunctions.Snowflake.GenerateSnowflake(epoch);
            Tables.Channels.Insert(
                Channels.ID.CreateParameter(chanSnowflake.ID),
                Channels.Parent.CreateParameter(ChanReq.Parent),
                Channels.Color.CreateParameter(Convert.FromHexString(ChanReq.Color)),
                Channels.Type.CreateParameter(ChanReq.Type),
                Channels.Epoch.CreateParameter(epoch.Date),
                Channels.Name.CreateParameter(Convert.FromBase64String(ChanReq.Name)),
                Channels.Description.CreateParameter(Convert.FromBase64String(ChanReq.Description)),
                Channels.RoleOverrides.CreateParameter(RoleOverides.ToArray()),
                Channels.UserOverrides.CreateParameter(UserOverides.ToArray()),
                Channels.TitleEncryptionKey.CreateParameter(ChanReq.TitleEncryptionKey),
                Channels.DescriptionEncryptionKey.CreateParameter(ChanReq.DescriptionEncryptionKey),
                Channels.EncryptionKeys.CreateParameter(ChanReq.EncryptionKeys),
                Channels.TitleEncoderType.CreateParameter(ChanReq.TitleEncoderType),
                Channels.DescriptionEncoderType.CreateParameter(ChanReq.DescriptionEncoderType),
                Channels.PictureType.CreateParameter(PictureType.none),
                Channels.Picture.CreateParameter(Array.Empty<byte>()),
                Channels.EncoderTypes.CreateParameter(ChanReq.EncoderTypes),
                Channels.Profiles.CreateParameter(Array.Empty<long>()));
            List<long> lll = Tables.Categories.Read(Categories.Channels, Categories.ID.CreateParameter(ChanReq.Parent))
                .ToList();
            lll.Add(chanSnowflake.ID);
            Tables.Categories.Update(Categories.ID, ChanReq.Parent, Categories.Channels.CreateParameter(lll.ToArray()));
            
            ChannelRow chan = Tables.Channels.ReadRow(Channels.ID.CreateParameter(chanSnowflake.ID));
            return this.ResponseToResult(new ChannelSTC()
            {
                Name = Convert.ToBase64String(chan.Name),
                Description = Convert.ToBase64String(chan.Description),
                ID = chanSnowflake.ID,
                Type = chan.Type,
                Parent = chan.Parent,
                Epoch = chan.Epoch,
                TitleEncryptionKey = chan.TitleEncryptionKey,
                DescriptionEncryptionKey = chan.DescriptionEncryptionKey,
                EncryptionKeys = chan.EncryptionKeys,
                TitleEncoderType = chan.TitleEncoderType,
                DescriptionEncoderType = chan.DescriptionEncoderType,
                EncoderTypes = chan.EncoderTypes,
                UserOverrides = chan.UserOverrides,
                RoleOverrides = chan.RoleOverrides,
                PictureType = chan.PictureType,
                Color = Convert.ToHexString(chan.Color)
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpDelete]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Delete([FromHeader(Name = "id")]long Channel)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!LuskiFunctions.HasAccessToCategory(ID, Channel, ServerPermission.DeleteChannels)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            Tables.Channels.DeleteRow(Channels.ID.CreateParameter(Channel));
            Tables.Messages.DeleteRow(Messages.ChannelID.CreateParameter(Channel));
            return StatusCode(StatusCodes.Status202Accepted);
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpPatch]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Patch([FromHeader(Name = "id")]long Channel)  //TODO Add Channel edit latter
    {
        return null;
    }
    
    [HttpGet]
    [Route(LuskiFunctions.Info.Routes.Default.Base + "/GetPicture")]
    public IActionResult GetPicture([FromHeader(Name = "id")]long chan)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!LuskiFunctions.HasAccessToChannel(ID, chan, ServerPermission.ViewThis)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            if (!Tables.Channels.TryRead(Channels.Picture, out byte[] image, Channels.ID.CreateParameter(chan))) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            return Tables.Channels.Read(Channels.PictureType, Channels.ID.CreateParameter(chan)) switch
            {
                PictureType.png => File(image, "image/png"),
                PictureType.jpeg => File(image, "image/jpeg"),
                PictureType.bmp => File(image, "image/bmp"),
                PictureType.gif => File(image, "image/gif"),
                PictureType.ico => File(image, "image/vnd.microsoft.icon"),
                PictureType.svg => File(image, "image/svg+xml"),
                PictureType.tif => File(image, "image/tiff"),
                PictureType.webp => File(image, "image/webp"),
                _ => File(image, "image/png"),
            };
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpGet]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Get([FromHeader(Name = "id")]long Channel)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!LuskiFunctions.HasAccessToChannel(ID, Channel, ServerPermission.ViewChannels)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            ChannelRow chan = Tables.Channels.ReadRow(Channels.ID.CreateParameter(Channel));
            return this.ResponseToResult(new ChannelSTC()
            {
                Name = Convert.ToBase64String(chan.Name),
                Description = Convert.ToBase64String(chan.Description),
                ID = Channel,
                Type = chan.Type,
                Parent = chan.Parent,
                Epoch = chan.Epoch,
                TitleEncryptionKey = chan.TitleEncryptionKey,
                DescriptionEncryptionKey = chan.DescriptionEncryptionKey,
                EncryptionKeys = chan.EncryptionKeys,
                TitleEncoderType = chan.TitleEncoderType,
                DescriptionEncoderType = chan.DescriptionEncoderType,
                EncoderTypes = chan.EncoderTypes,
                UserOverrides = chan.UserOverrides,
                RoleOverrides = chan.RoleOverrides,
                PictureType = chan.PictureType,
                Color = Convert.ToHexString(chan.Color)
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
}