using System.Net.Mime;
using Asp.Versioning;
using Luski.Shared.PublicServers.V1.Enums;
using Luski.Shared.PublicServers.V1.ServerToClient.HTTP;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class SocketChannelProfileController : ControllerBase
{
    /// <summary>
    /// Returns the Avatar for the channel profile.
    /// </summary>
    /// <param name="id">The ID of the requested profile</param>
    /// <returns></returns>
    [HttpGet]
    [ProducesResponseType(StatusCodes.Status200OK)]
    [ProducesResponseType(typeof(STC), StatusCodes.Status403Forbidden)]
    [Produces(MediaTypeNames.Image.Jpeg, MediaTypeNames.Image.Gif, MediaTypeNames.Image.Tiff, "image/png", MediaTypeNames.Application.Json)]
    [Route(LuskiFunctions.Info.Routes.Default.Base + "/Avatar/{id:long}")]
    public IActionResult? Avatar(long id)
    {
        if (Tables.ChannelProfiles.TryRead(ChannelProfiles.Picture, out byte[]? image, ChannelProfiles.ID.CreateParameter(id)))
        {
            return Tables.ChannelProfiles.Read(ChannelProfiles.PictureType, ChannelProfiles.ID.CreateParameter(id)) switch
            {
                PictureType.png => File(image, "image/png"),
                PictureType.jpeg => File(image, "image/jpeg"),
                PictureType.bmp => File(image, "image/bmp"),
                PictureType.gif => File(image, "image/gif"),
                PictureType.ico => File(image, "image/vnd.microsoft.icon"),
                PictureType.svg => File(image, "image/svg+xml"),
                PictureType.tif => File(image, "image/tiff"),
                PictureType.webp => File(image, "image/webp"),
                //should never happen
                _ => File(image, "image/png"),
            };
        }
        else
        {
            return StatusCode(403, new STC()
            {
                Error = ErrorCode.Forbidden,
                ErrorMessage = "the user you have given does not exist"
            });
        }
    }
    
    [HttpGet]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Get([FromHeader(Name = "id")]long Person)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (Person != ID && !Tables.ChannelProfiles.TryRead(ChannelProfiles.ID, out _, ChannelProfiles.ID.CreateParameter(Person))) return this.ShowError(ErrorCode.Forbidden, "Profile does not exist");
            return this.ResponseToResult(new ChannelProfileSTC()
            {
                ID = Person,
                DisplayName = Tables.ChannelProfiles.Read(ChannelProfiles.DisplayName, ChannelProfiles.ID.CreateParameter(Person)),
                PictureType = Tables.ChannelProfiles.Read(ChannelProfiles.PictureType, ChannelProfiles.ID.CreateParameter(Person)),
                Controllers = Tables.ChannelProfiles.Read(ChannelProfiles.Controllers, ChannelProfiles.ID.CreateParameter(Person)),
                Color = Convert.ToHexString(Tables.ChannelProfiles.Read(ChannelProfiles.Color, ChannelProfiles.ID.CreateParameter(Person))),
            });
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
}