using System.Net.Mime;
using Asp.Versioning;
using Luski.Shared.PublicServers.V1.ClientToServer.HTTP;
using Luski.Shared.PublicServers.V1.Enums;
using Luski.Shared.PublicServers.V1.ServerToClient.HTTP;
using LuskiServer.Classes;
using LuskiServer.Classes.ServerComs;
using LuskiServer.Classes.TableDef;
using LuskiServer.Enums.ServerComs;
using Microsoft.AspNetCore.Mvc;
using DataType = LuskiServer.Enums.ServerComs.DataType;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class SocketMessageController : ControllerBase
{
    private IActionResult GetMSG(long MSG_id)
    {
        MessageRow msgraw = Tables.Messages.ReadRow(Messages.ID.CreateParameter(MSG_id));
        MessageSTC msg = new()
        {
            AuthorID = msgraw.AuthorID,
            Context = Convert.ToBase64String(msgraw.Context),
            ID = MSG_id,
            ChannelID = msgraw.ChannelID
        };
        List<ServerFileInfoSTC> list = new();
        if (msgraw.Files.Any())
        {
            
            foreach (long b in msgraw.Files)
            {
                FileRow fileraw = Tables.Files.ReadRow(Files.ID.CreateParameter(b));
                ServerFileInfoSTC file = new()
                {
                    ID = b,
                    Name = Convert.ToBase64String(fileraw.Name),
                    Size = fileraw.Size,
                    NameEncoder = fileraw.NameEncoderType,
                    NameKey = fileraw.NameEncryptionKey,
                    Encoder = fileraw.EncoderType,
                    Key = fileraw.EncryptionKey
                };
                list.Add(file);
            }

            
        }
        msg.Files = list.ToArray();

        return this.ResponseToResult(msg);
    }
    
    [HttpGet]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Get([FromHeader(Name = "msg_id")]long MSG_id)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!Tables.Messages.TryRead(Messages.ChannelID, out long Channel_Id, Messages.ID.CreateParameter(MSG_id)) ||
                !LuskiFunctions.HasAccessToChannel(ID, Channel_Id, ServerPermission.ReadMessageHistory)) this.ResponseCodeToResult(ErrorCode.Forbidden);
            MessageRow msgraw = Tables.Messages.ReadRow(Messages.ID.CreateParameter(MSG_id));
            MessageSTC msg = new()
            {
                AuthorID = msgraw.AuthorID,
                Context = Convert.ToBase64String(msgraw.Context),
                ID = MSG_id,
                ChannelID = Channel_Id
            };
            if (msgraw.Files.Any())
            {
                List<ServerFileInfoSTC> list = new();
                foreach (long b in msgraw.Files)
                {
                    FileRow fileraw = Tables.Files.ReadRow(Files.ID.CreateParameter(b));
                    ServerFileInfoSTC file = new()
                    {
                        ID = b,
                        Name = Convert.ToBase64String(fileraw.Name),
                        Size = fileraw.Size,
                        NameEncoder = fileraw.NameEncoderType,
                        NameKey = fileraw.NameEncryptionKey,
                        Encoder = fileraw.EncoderType,
                        Key = fileraw.EncryptionKey
                    };
                    list.Add(file);
                }

                msg.Files = list.ToArray();
            }

            return this.ResponseToResult(msg);
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    [HttpPost]
    [DisableRequestSizeLimit]
    [Consumes(typeof(MessageCTS), MediaTypeNames.Application.Json)]
    [Route(LuskiFunctions.Info.Routes.Default.Base)]
    public IActionResult Post([FromBody]MessageCTS data)
    {
        try
        {
            if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
            if (!LuskiFunctions.HasAccessToChannel(ID, data.ChannelID, ServerPermission.SendMessages)) return this.ResponseCodeToResult(ErrorCode.Forbidden);
            LuskiFunctions.Snowflake Id = LuskiFunctions.Snowflake.GenerateSnowflake(Tables.Channels.Read(Channels.Epoch, Channels.ID.CreateParameter(data.ChannelID)));
            ChannelType type = Tables.Channels.Read(Channels.Type, Channels.ID.CreateParameter(data.ChannelID));
            bool Fake = data.Profile.HasValue;
            if (Fake)
            {
                ID = data.Profile!.Value;
            }
            MessageEvent MessageEvent = new()
            {
                UserID = ID,
                ChannelID = data.ChannelID,
                Base64Context = data.Base64Context,
                ID = Id.ID,
                EncoderType = data.Encoding,
                EncryptionKey = data.EncryptionKey,
                Files = Array.Empty<ServerFileInfoSTC>(),
                IsProfile = Fake,
            };
            Tables.Messages.Insert(
                Messages.ChannelID.CreateParameter(data.ChannelID),
                Messages.ID.CreateParameter(Id.ID),
                Messages.AuthorID.CreateParameter(ID),
                Messages.Context.CreateParameter(Convert.FromBase64String(data.Base64Context)),
                Messages.Files.CreateParameter(data.Files),
                Messages.EncoderType.CreateParameter(data.Encoding),
                Messages.EncryptionKey.CreateParameter(data.EncryptionKey),
                Messages.ReplyTo.CreateParameter(Id.ID),
                Messages.IsChannelProfile.CreateParameter(Fake));
            WSS.SendData(SendType.ID_Group, new ServerEvent()
            {
                Type = DataType.MessageCreate,
                Data = MessageEvent
            }, LuskiFunctions.GetChannelBroadcastMembers(data.ChannelID));
            Response.StatusCode = 201;
            return GetMSG(Id.ID);
        }
        catch (Exception e)
        {
            return this.ShowError(e);
        }
    }
    
    //TODO Add Message edit latter
}