using Asp.Versioning;
using Luski.Shared.PublicServers.V1.Enums;
using Luski.Shared.PublicServers.V1.ServerToClient.HTTP;
using Luski.Shared.PublicServers.V1.Shared;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class SocketOverridesController : ControllerBase
{
    [HttpGet]
    [Route(LuskiFunctions.Info.Routes.Default.Base + "/UserOverride/{id:long}")]
    public IActionResult GetUserOverride([FromQuery(Name = "id")]long id)
    {
        if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
        if (!Tables.UserRoleOverrides.TryReadRow(out UserRoleOverrideRow row, UserRoleOverrides.ID.CreateParameter(id))) return this.ShowError(ErrorCode.Forbidden, "Override does not exist");
        return this.ResponseToResult(new UserOverrideSTC()
        {
            Id = row.ID,
            UserID = row.UserID,
            GoodPermissions = row.GoodPermissions,
            BadPermissions = row.BadPermissions
        });
    }
    
    [HttpGet]
    [Route(LuskiFunctions.Info.Routes.Default.Base + "/RoleOverride/{id:long}")]
    public IActionResult GetRoleOverride([FromQuery(Name = "id")]long id)
    {
        if (!this.CanTokenRequest(out long ID, out long SID, out IActionResult? toc) && toc != null) return toc;
        if (!Tables.ServerRoleOverrides.TryReadRow(out ServerRoleOverrideRow? row, ServerRoleOverrides.ID.CreateParameter(id))) return this.ShowError(ErrorCode.Forbidden, "Override does not exist");
        return this.ResponseToResult(new UserRoleOverrideSTC()
        {
            Id = row.ID,
            RoleID = row.RoleID,
            GoodPermissions = row.GoodPermissions,
            BadPermissions = row.BadPermissions
        });
    }
}