using System.IO.Compression;
using System.Text.Json;
using JacobTechEncryption.Enums;
using Luski.Shared.PublicServers.V1.Enums;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using LuskiServer.Converters.Matrix.Types;

namespace LuskiServer.Converters.Matrix;

public static class Matrix
{
    public static void ZipJsonChatExport(Stream Zip, long Channel, Dictionary<string, long> UserMap,
        Dictionary<string, int> ChannelProfilesIdList,
        List<Tuple<string, byte[], PictureType, byte[]>> Profiles,
        List<string> IgnoredUsers, EncoderType Encoder, long Encryption, bool remove = true)
    {
        Dictionary<int, long> IdConverter = new();
        Dictionary<string, long> cps = new();
        if (Tables.Channels.TryRead(Channels.ID, out _, Channels.ID.CreateParameter(Channel)))
        {
            Tables.Messages.DeleteRow(Messages.ChannelID.CreateParameter(Channel));
            //throw new Exception("Channel Exist");
        }
        
        using ZipArchive archive = new ZipArchive(Zip, ZipArchiveMode.Read, false);
        string internalfolder = archive.Entries[0].FullName;
        ZipArchiveEntry json = archive.GetEntry(internalfolder + "export.json")!;
        MessageExport export = JsonSerializer.Deserialize(json.Open(), MessageExportContext.Default.MessageExport)!;

        void checkFakeUser(string id)
        {
            if (!IdConverter.ContainsKey(ChannelProfilesIdList[id]))
            {
                long id2 = LuskiFunctions.Snowflake.GenerateSnowflake().ID;
                        
                Tables.ChannelProfiles.Insert(
                    ChannelProfiles.ID.CreateParameter(id2),
                    ChannelProfiles.Controllers.CreateDefaultParameter(),
                    ChannelProfiles.PictureType.CreateParameter(Profiles[ChannelProfilesIdList[id]].Item3),
                    ChannelProfiles.Picture.CreateParameter(Profiles[ChannelProfilesIdList[id]].Item2),
                    ChannelProfiles.DisplayName.CreateParameter(Profiles[ChannelProfilesIdList[id]].Item1),
                    ChannelProfiles.Color.CreateParameter(Profiles[ChannelProfilesIdList[id]].Item4));
                IdConverter.Add(ChannelProfilesIdList[id], id2);
                foreach (KeyValuePair<string, int> v in ChannelProfilesIdList.Where(s => s.Value == ChannelProfilesIdList[id]))
                {
                    cps.Add(v.Key, id2);
                }
            }
        }

        DateTime epoch = DateTime.UnixEpoch.AddMilliseconds(export.Messages[0].Origin).Date;

        try
        {
            //TODO REMOVE FOR LAST VERSION
            foreach (long i in Tables.Channels.Read(Channels.Profiles, Channels.ID.CreateParameter(Channel)))
            {
                Tables.ChannelProfiles.DeleteRow(ChannelProfiles.ID.CreateParameter(i));
            }
            
            Tables.Channels.Update(Channels.ID, Channel,
                //Tables.Channels.Insert(
                Channels.ID.CreateParameter(Channel),
                Channels.Parent.CreateParameter(0),
                Channels.Type.CreateParameter(ChannelType.TextAndVoice),
                Channels.Description.CreateParameter(export.RoomTopic.ToDB(Encoder, Encryption)),
                Channels.Name.CreateParameter(export.RoomName.ToDB(Encoder, Encryption)),
                Channels.RoleOverrides.CreateParameter(Array.Empty<long>()),
                Channels.UserOverrides.CreateParameter(Array.Empty<long>()),
                Channels.Epoch.CreateParameter(epoch),
                Channels.TitleEncryptionKey.CreateParameter(Encryption),
                Channels.DescriptionEncryptionKey.CreateParameter(Encryption),
                Channels.EncryptionKeys.CreateParameter(new long[] { Encryption }),
                Channels.TitleEncoderType.CreateParameter(Encoder),
                Channels.DescriptionEncoderType.CreateParameter(Encoder),
                //Channels.PictureType.CreateParameter(PictureType.none),
                //Channels.Picture.CreateParameter(Array.Empty<byte>()),
                Channels.EncoderTypes.CreateParameter(new[] { Encoder }));

            Dictionary<string, long> Events = new();
            foreach (Message msg in export.Messages)
            {
                if (msg.Type != "m.room.message")
                    continue;
                if (!UserMap.ContainsKey(msg.Sender) && !ChannelProfilesIdList.ContainsKey(msg.Sender))
                {
                    if (IgnoredUsers.Contains(msg.Sender))
                        continue;
                    if (remove)
                    {
                        Tables.Messages.DeleteRow(Messages.ChannelID.CreateParameter(Channel));
                    }

                    throw new Exception($"User not in list: {msg.Sender}");
                }


                DateTime cur = DateTime.UnixEpoch.AddMilliseconds(msg.Origin);
                if (msg.Context.Type is null)
                    continue;
                LuskiFunctions.Snowflake sf = LuskiFunctions.Snowflake.GenerateSnowflake(epoch, cur);
                long reply = sf.ID;
                string body = msg.Context.Body;
                
                Events.Add(msg.Event, sf.ID);
                cur = cur.ToLocalTime();
                bool chanp;
                long id;
                if (ChannelProfilesIdList.ContainsKey(msg.Sender))
                {
                    chanp = true;
                    checkFakeUser(msg.Sender);
                    id = IdConverter[ChannelProfilesIdList[msg.Sender]];
                }
                else
                {
                    chanp = false;
                    id = UserMap[msg.Sender];
                }
                
                if (msg.Context.Relations is not null)
                {
                    if (Events.TryGetValue(msg.Context.Relations.Reply.ID, out long greply)) reply = greply;
                    //Console.WriteLine("Body: {0}", msg.Context.Body);
                    //Console.WriteLine("Fancy Body: {0}", msg.Context.FormattedBody);
                    // string[] ars = msg.Context.FormattedBody.Split('\n');
                    if (msg.Context.FormattedBody is null) body = msg.Context.Body;
                    else body = msg.Context.FormattedBody.Split("</mx-reply>")[^1]
                        .Replace("<br/>", "\n")
                        .Replace("<br>", "\n")
                        .Replace("&#x27;", "'")
                        .Replace("&#39;", "'")
                        .Replace("\\", "\\\\")
                        .Replace("[", "\\[");
                    if (body.Contains("could build a better dropdown than Discord"))
                    {
                        Console.WriteLine("break");
                    }
                    
                    if (msg.Context.Mentions is not null && msg.Context.Mentions.IDs is not null && msg.Context.Mentions.IDs.Length > 0)
                    {
                        foreach (var i in msg.Context.Mentions.IDs)
                        {
                            string test = "[";
                            if (UserMap.ContainsKey(i))
                            {
                                test += $"@{UserMap[i]}]";
                            }
                            else
                            {
                                checkFakeUser(i);
                                test += $"%{cps[i]}>";
                            }
                            body = body.Replace($"<a href=\"https://matrix.to/#/{i}\"",test);
                            int index = -1;
                            while (true)
                            {
                                index = body.IndexOf(test, index + 1, StringComparison.Ordinal);
                                if (index == -1) break;
                                int index2 = body.IndexOf("</a>", index + 1, StringComparison.Ordinal);
                                body = body.Remove(index + test.Length, index2 + 4 - test.Length - index);
                            }
                        }
                    }

                    body = body.Replace("<p>", "").Replace("</p>", "\n");
                    if (body.EndsWith("\n")) body = body.Remove(body.Length - 1, 1);
                    //body = msg.Context.FormattedBody;
                    Console.WriteLine(body);
                }
                else
                {
                    if (msg.Context.Mentions is not null && msg.Context.Mentions.IDs is not null && msg.Context.Mentions.IDs.Length > 0)
                    {
                        body = msg.Context.FormattedBody!;
                        foreach (var i in msg.Context.Mentions.IDs)
                        {
                            string test = "[";
                            if (UserMap.ContainsKey(i))
                            {
                                test += $"@{UserMap[i]}>";
                            }
                            else
                            {
                                checkFakeUser(i);
                                test += $"%{cps[i]}]";
                            }
                            body = body!.Replace($"<a href=\"https://matrix.to/#/{i}\"",test);
                            int index = -1;
                            while (true)
                            {
                                index = body.IndexOf(test, index + 1, StringComparison.Ordinal);
                                if (index == -1) break;
                                int index2 = body.IndexOf("</a>", index + 1, StringComparison.Ordinal);
                                body = body.Remove(index + test.Length, index2 + 4 - test.Length - index);
                            }
                        }
                        body = body!.Replace("<p>", "").Replace("</p>", "\n");
                        if (body.EndsWith("\n")) body = body.Remove(body.Length - 1, 1);
                    }
                }
                
                switch (msg.Context.Type)
                {
                    default:
                        Tables.Messages.Insert(
                            Messages.ID.CreateParameter(sf.ID),
                            Messages.Context.CreateParameter(body.ToDB(Encoder, Encryption)),
                            Messages.ChannelID.CreateParameter(Channel),
                            Messages.EncoderType.CreateParameter(Encoder),
                            Messages.Files.CreateParameter(Array.Empty<long>()),
                            Messages.EncryptionKey.CreateParameter(Encryption),
                            Messages.AuthorID.CreateParameter(id),
                            Messages.IsChannelProfile.CreateParameter(chanp),
                            Messages.ReplyTo.CreateParameter(reply));
                        break;
                    case "m.image" or "m.video" or "m.audio" or "m.file":
                        int h = cur.Hour;
                        string pm = "AM";
                        if (h > 11)
                        {
                            h = h - 12;
                            if (h == 0) h = 12;
                            pm = "PM";
                        }

                        string hh = h.ToString();
                        string mm = cur.Minute.ToString();
                        string s = cur.Second.ToString();
                        if (s.Length == 1) s = "0" + s;
                        if (mm.Length == 1) mm = "0" + mm;
                        if (hh.Length == 1) hh = "0" + hh;
                        string[] spl = msg.Context.Body.Split('.');
                        string fil =
                            $"{spl[0]}-{cur.Month}-{cur.Day}-{cur.Year} at {h}-{mm}-{s} {pm}.{spl[spl.Length - 1]}";
                        string loc = msg.Context.Type.Split('.')[1] + "s";
                        if (loc == "audios") loc = "audio";
                        if (fil.EndsWith(".1")) fil = fil.Remove(fil.Length - 2, 2);
                        ZipArchiveEntry? f;
                        if ((f = archive.GetEntry(internalfolder + loc + "/" + fil)) is not null)
                        {
                            byte[] d;
                            using(var memoryStream = new MemoryStream())
                            {
                                f.Open().CopyTo(memoryStream);
                                d = memoryStream.ToArray();
                            }

                            Tables.Files.Insert(
                                Files.ID.CreateParameter(sf.ID),
                                Files.Name.CreateParameter(msg.Context.Body.ToDB(Encoder, Encryption)),
                                Files.Data.CreateParameter(d),
                                Files.EncoderType.CreateParameter(Encoder),
                                Files.EncryptionKey.CreateParameter(Encryption),
                                Files.Hash.CreateParameter(JacobTechEncryption.Encryption.Hashing.SHA256(d)),
                                Files.Owner.CreateParameter(id),
                                Files.AllowedChannels.CreateParameter(new long[] { Channel }),
                                Files.Public.CreateParameter(true),
                                Files.Size.CreateParameter(d.LongLength),
                                Files.NameEncoderType.CreateParameter(Encoder),
                                Files.NameEncryptionKey.CreateParameter(Encryption));
                            Tables.Messages.Insert(
                                Messages.ID.CreateParameter(sf.ID),
                                Messages.Context.CreateParameter(Array.Empty<byte>()),
                                Messages.ChannelID.CreateParameter(Encryption),
                                Messages.EncoderType.CreateParameter(Encoder),
                                Messages.Files.CreateParameter(new[] { sf.ID }),
                                Messages.EncryptionKey.CreateParameter(Encryption),
                                Messages.AuthorID.CreateParameter(id),
                                Messages.IsChannelProfile.CreateParameter(chanp),
                                Messages.ReplyTo.CreateParameter(reply));
                        }
                        break;
                }
            }

            Tables.Channels.Update(Channels.ID, Channel, Channels.Profiles.CreateParameter(IdConverter.Values.ToArray()));
        }
        catch (Exception e)
        {
            Console.WriteLine(e);
            if (remove)
            {
                Tables.Messages.DeleteRow(Messages.ChannelID.CreateParameter(Channel));
            }
            throw;
        }
    }
}